import base64
import hashlib
import hmac
import requests
import time
import urllib

from krake.config import load_config
from krake.error import APIError
from krake.response import APIResponse


class API:
    """API implements the kraken v0 api as specifiyed at https://www.kraken.com/help/api.

    Attributes:
        key: Kraken.com api key.
        secret: Kraken.com api secret.
        timeout: Request timeout in seconds.

    """

    def __init__(self, key=None, secret=None, timeout=None):
        """Create an API instance and try to load the values for key, secret and timeout if they
        are not specifyed explizitly in the constructor.

        Args:
            key: Kraken.com api key.
            secret: Kraken.com api secret.
            timeout: Request timeout in seconds.

        """
        config = load_config(key=key, secret=secret, timeout=timeout)
        self.key, self.secret, self.timeout = config['key'], config['secret'], config['timeout']

        self._base_url = 'https://api.kraken.com'
        self._session = requests.Session()
        self._session.headers.update({'api-key': self.key})

    def __del__(self):
        self._session.close()

    def _sign(self, payload, path):
        hashed = hashlib.sha256(
            (str(payload['nonce']) + urllib.parse.urlencode(payload)).encode()).digest()
        message = path.encode() + hashed
        signature = hmac.new(base64.b64decode(self.secret), message, hashlib.sha512)
        return base64.b64encode(signature.digest()).decode()

    def _request_wrapper(func):
        def wrapper(self, method, payload={}):
            start = time.time()
            resp = func(self, method, payload)
            end = time.time()

            json_resp = resp.json()
            if 'error' in json_resp and len(json_resp['error']) > 0:
                raise APIError(json_resp['error'])

            meta = dict(url=resp.request.url,
                        method=method,
                        time=start,
                        response_time=int((end-start)*1e3))
            return APIResponse(json_resp['result'], meta)
        return wrapper

    @_request_wrapper
    def _public_post(self, method, payload={}):
        path = '/0/public/' + method

        return self._session.post(self._base_url + path, data=payload, timeout=self.timeout)

    @_request_wrapper
    def _private_post(self, method, payload={}):
        if self.key is None or self.secret is None:
            raise APIError('No api credentials provided.')

        path = '/0/private/' + method
        payload['nonce'] = int(time.time()*1e6)

        return self._session.post(self._base_url + path, data=payload,
                                  headers={'api-sign': self._sign(payload, path)},
                                  timeout=self.timeout)

    def time(self):
        """Get server time.

        """
        return self._public_post('Time')

    def assets(self, **payload):
        """Get asset info.

        """
        return self._public_post('Assets', payload)

    def asset_pairs(self, **payload):
        """Get tradable asset pairs.

        """
        return self._public_post('AssetPairs')

    def ticker(self, **payload):
        """Get ticker information.

        Args:
            pair: Single asset pair or list of asset pairs.

        """
        return self._public_post('Ticker', payload)

    def ohlc(self, **payload):
        """Get OHLC data.

        Args:
            pair: Asset pair to get OHLC data for.
            interval: Time frame interval in minutes, default 1.
            since: Return committed OHLC data since given id.

        """
        return self._public_post('OHLC', payload)

    def depth(self, **payload):
        """Get order book.

        Args:
            pair: Asset pair to get market depth for.
            count: Maximum number of asks/bids.

        """
        return self._public_post('Depth', payload)

    def trades(self, **payload):
        """Get recent trades.

        Args:
            pair: Asset pair to get trade data for.
            since: Return trade data since given id.

        """
        return self._public_post('Trades', payload)

    def spread(self, **payload):
        """Get recent spread data.

        Args:
            pair: Asset pair to get spread data for.
            since: Return spread data since given id.

        """
        return self._public_post('Spread', payload)

    def balance(self):
        """Get account balance.

        """
        return self._private_post('Balance')

    def trade_balance(self, **payload):
        """Get trade balance.

        """
        return self._private_post('TradeBalance', payload)

    def open_orders(self, **payload):
        """Get open orders.

        """
        return self._private_post('OpenOrders', payload)

    def closed_orders(self, **payload):
        """Get closed orders.

        """
        return self._private_post('ClosedOrders', payload)

    def query_orders(self, **payload):
        """Query orders info.

        """
        return self._private_post('QueryOrders', payload)

    def trades_history(self, **payload):
        """Get trades history.

        """
        return self._private_post('TradesHistory', payload)

    def query_trades(self, **payload):
        """Query trades info.

        """
        return self._private_post('QueryTrades', payload)

    def open_positions(self, **payload):
        """Get open positions.

        """
        return self._private_post('OpenPositions', payload)

    def ledgers(self, **payload):
        """Get ledgers info.

        """
        return self._private_post('Ledgers', payload)

    def query_ledgers(self, **payload):
        """Query ledgers.

        """
        return self._private_post('QueryLedgers', payload)

    def trade_volume(self, **payload):
        """Get trade volume.

        """
        return self._private_post('TradeVolume', payload)

    def add_order(self, **payload):
        """Add standard order.

        """
        return self._private_post('AddOrder', payload)

    def cancel_order(self, **payload):
        """Cancel open order.

        """
        return self._private_post('CancelOrder', payload)

    def deposit_methods(self, **payload):
        """Get deposit methods.

        """
        return self._private_post('DepositMethods', payload)

    def deposit_addresses(self, **payload):
        """Get deposit addresses.

        """
        return self._private_post('DepositAddresses', payload)

    def deposit_status(self, **payload):
        """Get status of recent deposits.

        """
        return self._private_post('DepositStatus', payload)

    def withdraw_info(self, **payload):
        """Get withdrawal information.

        """
        return self._private_post('WithdrawInfo', payload)

    def withdraw(self, **payload):
        """Withdraw funds.

        """
        return self._private_post('Withdraw', payload)

    def withdraw_status(self, **payload):
        """Get status of recent withdrawals.

        """
        return self._private_post('WithdrawStatus', payload)

    def withdraw_cancel(self, **payload):
        """Request withdrawal cancelation.

        """
        return self._private_post('WithdrawCancel', payload)
