# -*- coding: utf-8 -*-
from __future__ import print_function

import click
import json
import os
import requests
import style
import time
import yaml

from distutils.version import StrictVersion
from krake import __version__
from krake.api import API


_help_description = 'Show help.'
_json_description = 'Output response in json format.'
_config_file_name = os.path.expanduser('~/.config/krake.yml')

_dim = style.dim
_red = style.light_red
_green = style.light_green


@click.group()
@click.help_option('-h', '--help', help=_help_description)
@click.version_option(__version__, '-V', '--version', message='%(version)s')
def cli():
    """Interact with the 🐙 kraken.com bitcoin exchange API."""
    check_package_version()


@cli.command()
@click.help_option('-h', '--help', help=_help_description)
@click.option('--key')
@click.option('--secret')
def config(key, secret):
    """Manage API credentials."""
    if os.path.exists(_config_file_name):
        with open(_config_file_name, 'r') as file:
            config = yaml.load(file)

        print('Configuration (at %s):' % _config_file_name)
        for key in ['key', 'secret', 'timeout']:
            print('', _dim('%-7s' % key), config.get(key, style.yellow('missing')), sep='  ')
    else:
        print(style.yellow.bold('No configuration provided.'))
        print()
        print('Create a configuration file:')
        print('  $ touch %s' % _config_file_name)
        print('  $ echo "key: <api key>" >> %s' % _config_file_name)
        print('  $ echo "secret: <api secret>" >> %s' % _config_file_name)


@cli.command()
@click.help_option('-h', '--help', help=_help_description)
@click.option('--json', 'output_json', is_flag=True, help=_json_description)
def balance(output_json):
    """List all currency balances."""
    balance = API().balance()
    if output_json:
        print_json(balance)
        return

    print('Balance:')
    for currency, balance in sorted(balance.items()):
        if currency == 'meta':
            continue
        print('  %3s %g' % (_dim(currency[1:]), float(balance)))


@cli.command()
@click.help_option('-h', '--help', help=_help_description)
@click.option('--json', 'output_json', is_flag=True, help=_json_description)
def orders(output_json):
    """List all orders."""
    api = API()
    open_orders = api.open_orders()
    closed_orders = api.closed_orders()
    if output_json:
        open_orders['closed'] = closed_orders['closed']
        print_json(open_orders)
        return

    open_orders = open_orders['open']
    closed_orders = closed_orders['closed']
    sorted_open_orders = sorted(open_orders.items(), key=lambda item: item[1]['opentm'],
                                reverse=True)
    sorted_closed_orders = sorted(closed_orders.items(), key=lambda item: item[1]['opentm'],
                                  reverse=True)

    if len(open_orders) > 0:
        print('%d open order%s:' % (len(open_orders), 's' if len(open_orders) != 1 else ''))
        print('  Opening time            Type          Pair      Price   Volume  Cost')
        for oid, order in sorted_open_orders:
            time_str = format_time(order['opentm'])
            type = _green('Buy ') if order['descr']['type'] == 'buy' else _red('Sell')
            order_type = _dim('%-6s' % order['descr']['ordertype'])
            pair = _dim('%-8s' % order['descr']['pair'])
            price = '%6.1f' % float(order['descr']['price'])
            volume = order['vol']
            cost = '%6.1f' % float(order['cost'])
            print('', _dim(time_str), type, order_type, pair, price, volume, cost, sep='  ')
    else:
        print('No open orders.')

    if len(closed_orders) > 0:
        print()
        print('%d closed order%s:' % (len(closed_orders), 's' if len(closed_orders) != 1 else ''))
        print('  Opening time            Type          Pair      Price   Volume      Cost  ' +
              '  Status')
        for oid, order in sorted_closed_orders:
            time_str = format_time(order['opentm'])
            type = _green('Buy ') if order['descr']['type'] == 'buy' else _red('Sell')
            order_type = _dim('%-6s' % order['descr']['ordertype'])
            pair = _dim('%-8s' % order['descr']['pair'])
            price = '%6.1f' % float(order['descr']['price'])
            volume = order['vol']
            cost = '%6.1f' % float(order['cost'])
            status = order['status'] if order['status'] == 'closed' else _dim(order['status'])
            print('', _dim(time_str), type, order_type, pair, price, volume, cost, status,
                  sep='  ')
    else:
        print('No closed orders.')


@cli.command()
@click.help_option('-h', '--help', help=_help_description)
# @click.option('--limit', default=10, help='maximum number of trades to display (default 10)')
@click.option('--json', 'output_json', is_flag=True, help=_json_description)
def trades(output_json):
    """List all trades."""
    trades = API().trades_history()
    if output_json:
        print_json(trades)
        return

    trades = trades['trades']
    sorted_trades = sorted(trades.items(), key=lambda item: item[1]['time'], reverse=True)

    if len(trades) > 0:
        print('%d trade%s:' % (len(trades), 's' if len(trades) != 1 else ''))
        print('  Time                    Type          Pair     Price   Volume      Fee')
        for tid, trade in sorted_trades:
            time_str = format_time(trade['time'])
            order = _green('Buy ') if trade['type'] == 'buy' else _red('Sell')
            order_type = _dim('%-6s' % trade['ordertype'])
            pair = _dim('%3s %3s' % (trade['pair'][1:4], trade['pair'][5:8]))
            price = '%6.1f' % float(trade['price'])
            volume = trade['vol']
            fee = _dim(trade['fee'])
            print('', _dim(time_str), order, order_type, pair, price, volume, fee, sep='  ')
    else:
        print('No trades.')


@cli.command()
@click.help_option('-h', '--help', help=_help_description)
@click.option('--json', 'output_json', is_flag=True, help=_json_description)
def buy(output_json):
    """Create a buy order."""

    raise NotImplementedError


@cli.command()
@click.help_option('-h', '--help', help=_help_description)
@click.option('--json', 'output_json', is_flag=True, help=_json_description)
def sell(output_json):
    """Create a sell order."""

    raise NotImplementedError


def check_package_version():
    """Check if the package is up to date and print upgrade instructions if a newer version is
    available.

    """
    try:
        package_info = requests.get('https://pypi.python.org/pypi/krake/json', timeout=.5).json()
        newest_version = StrictVersion(package_info['info']['version'])
        current_version = StrictVersion(__version__)
        if newest_version > current_version:
            print(style.yellow('A new version of', style.bold('krake'), 'is available. Run',
                               style.bold('pip install -U krake'), 'to upgrade.'), end='\n'*2)
    except:
        pass


def print_json(dict):
    """Print a dictionary as compact json.

    Args:
        dict: dictionary to print.

    """
    print(json.dumps(dict, sort_keys=True, separators=(',', ':')))


def format_time(millis):
    return time.strftime('%b %d, %Y, %H:%M:%S', time.localtime(millis))


if __name__ == '__main__':
    cli()
