import os
import re
import yaml


def load_config(**kwargs):
    """Load values for given keys from different sources.

    Trys to load values for given keys form different sources in the following order:
        1. The provided value for the key, if not None.
        2. An environment variable of format KRAKE_<UPPERCASE_KEY>
        3. From a config file at ./krake.yml
        4. From a config file at ~/.config/krake.yml

    Args:
        **kwargs: Keyword arguments for which values are looked up.
    Returns:
        A dictionary with the looked up values.
    Raises:
        ValueError: If a key in kwargs is not lower_case_with_underscores.

    """
    for key, value in kwargs.items():
        if not _check_key_name(key):
            raise ValueError('Keys must be lower_case_with_underscores, not %s.' % key)

        if value is not None:
            kwargs[key] = value
        elif _os_env_name(key) in os.environ:
            kwargs[key] = os.environ[_os_env_name(key)]
        else:
            for config_file_name in _config_file_names():
                if os.path.exists(os.path.expanduser(config_file_name)):
                    with open(os.path.expanduser(config_file_name)) as file:
                        config = yaml.load(file)
                    if key in config and config[key] is not None:
                        kwargs[key] = config[key]
                        break
    return kwargs


def _check_key_name(name):
    return re.match('^([a-z]+_)*[a-z]+$', name) is not None


def _package_name():
    return 'krake'


def _os_env_name(name):
    return '%s_%s' % (_package_name().upper(), name.upper())


def _config_file_names():
    return ['%s.yml' % _package_name(), '~/.config/%s.yml' % _package_name()]
